@@ -20,12 +20,15 @@
  * Author: Alexander Larsson <alexl@redhat.com>
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <glib.h>
 #include <gfileoutputstream.h>
 #include <gseekable.h>
 #include "gsimpleasyncresult.h"
+#include "gasyncresult.h"
+#include "gcancellable.h"
+#include "gioerror.h"
 #include "glibintl.h"
 
 #include "gioalias.h"
@@ -42,15 +45,14 @@
  * GFileOutputStream implements #GSeekable, which allows the output 
  * stream to jump to arbitrary positions in the file and to truncate
  * the file, provided the filesystem of the file supports these 
- * operations. In addition to the generic g_seekable_ API, 
- * GFileOutputStream has its own API for seeking and positioning. 
- * To find the position of a file output stream, use 
- * g_file_output_stream_tell(). To find out if a file output 
- * stream supports seeking, use g_file_output_stream_can_seek().
- * To position a file output stream, use g_file_output_stream_seek().
- * To find out if a file output stream supports truncating, use
- * g_file_output_stream_can_truncate(). To truncate a file output
- * stream, use g_file_output_stream_truncate().
+ * operations.
+ *
+ * To find the position of a file output stream, use g_seekable_tell().
+ * To find out if a file output stream supports seeking, use
+ * g_seekable_can_seek().To position a file output stream, use
+ * g_seekable_seek(). To find out if a file output stream supports
+ * truncating, use g_seekable_can_truncate(). To truncate a file output
+ * stream, use g_seekable_truncate().
  **/
 
 static void       g_file_output_stream_seekable_iface_init    (GSeekableIface       *iface);
@@ -67,7 +69,7 @@ static gboolean   g_file_output_stream_seekable_truncate      (GSeekable
 							       GCancellable         *cancellable,
 							       GError              **error);
 static void       g_file_output_stream_real_query_info_async  (GFileOutputStream    *stream,
-							       char                 *attributes,
+							       const char           *attributes,
 							       int                   io_priority,
 							       GCancellable         *cancellable,
 							       GAsyncReadyCallback   callback,
@@ -140,7 +142,7 @@ g_file_output_stream_init (GFileOutputStream *stream)
  **/
 GFileInfo *
 g_file_output_stream_query_info (GFileOutputStream      *stream,
-				    char                   *attributes,
+				    const char             *attributes,
 				    GCancellable           *cancellable,
 				    GError                **error)
 {
@@ -164,8 +166,8 @@ g_file_output_stream_query_info (GFileOutputStream      *stream,
   if (class->query_info)
     info = class->query_info (stream, attributes, cancellable, error);
   else
-    g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-		 _("Stream doesn't support query_info"));
+    g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+                         _("Stream doesn't support query_info"));
   
   if (cancellable)
     g_cancellable_pop_current (cancellable);
@@ -208,7 +210,7 @@ async_ready_callback_wrapper (GObject *source_object,
  **/
 void
 g_file_output_stream_query_info_async (GFileOutputStream     *stream,
-					  char                 *attributes,
+					  const char           *attributes,
 					  int                   io_priority,
 					  GCancellable         *cancellable,
 					  GAsyncReadyCallback   callback,
@@ -377,8 +379,8 @@ g_file_output_stream_seek (GFileOutputStream  *stream,
 
   if (!class->seek)
     {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-		   _("Seek not supported on stream"));
+      g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+                           _("Seek not supported on stream"));
       return FALSE;
     }
 
@@ -453,8 +455,8 @@ g_file_output_stream_truncate (GFileOutputStream  *stream,
 
   if (!class->truncate_fn)
     {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-		   _("Truncate not supported on stream"));
+      g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+                           _("Truncate not supported on stream"));
       return FALSE;
     }
 
@@ -519,8 +521,8 @@ query_info_async_thread (GSimpleAsyncResult *res,
   if (class->query_info)
     info = class->query_info (G_FILE_OUTPUT_STREAM (object), data->attributes, cancellable, &error);
   else
-    g_set_error (&error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-		 _("Stream doesn't support query_info"));
+    g_set_error_literal (&error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+                         _("Stream doesn't support query_info"));
 
   if (info == NULL)
     {
@@ -533,7 +535,7 @@ query_info_async_thread (GSimpleAsyncResult *res,
 
 static void
 g_file_output_stream_real_query_info_async (GFileOutputStream     *stream,
-					       char                 *attributes,
+					       const char           *attributes,
 					       int                   io_priority,
 					       GCancellable         *cancellable,
 					       GAsyncReadyCallback   callback,
