@@ -17,7 +17,9 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * Author: Christian Kellner <gicmo@gnome.org>
+ * Authors:
+ *   Christian Kellner <gicmo@gnome.org>
+ *   Krzysztof Kosiński <tweenk.pl@gmail.com>
  */
 
 #include "config.h"
@@ -44,18 +46,35 @@
 
 #define MIN_ARRAY_SIZE  16
 
+enum {
+  PROP_0,
+  PROP_DATA,
+  PROP_SIZE,
+  PROP_DATA_SIZE,
+  PROP_REALLOC_FUNCTION,
+  PROP_DESTROY_FUNCTION
+};
+
 struct _GMemoryOutputStreamPrivate {
   
-  gpointer       data;
-  gsize          len;
+  gpointer       data; /* Write buffer */
+  gsize          len; /* Current length of the data buffer. Can change with resizing. */
   gsize          valid_len; /* The part of data that has been written to */
-
-  goffset        pos;
+  gsize          pos; /* Current position in the stream. Distinct from valid_len,
+                         because the stream is seekable. */
 
   GReallocFunc   realloc_fn;
   GDestroyNotify destroy;
 };
 
+static void     g_memory_output_stream_set_property (GObject      *object,
+                                                     guint         prop_id,
+                                                     const GValue *value,
+                                                     GParamSpec   *pspec);
+static void     g_memory_output_stream_get_property (GObject      *object,
+                                                     guint         prop_id,
+                                                     GValue       *value,
+                                                     GParamSpec   *pspec);
 static void     g_memory_output_stream_finalize     (GObject      *object);
 
 static gssize   g_memory_output_stream_write       (GOutputStream *stream,
@@ -115,7 +134,9 @@ g_memory_output_stream_class_init (GMemoryOutputStreamClass *klass)
   g_type_class_add_private (klass, sizeof (GMemoryOutputStreamPrivate));
 
   gobject_class = G_OBJECT_CLASS (klass);
-  gobject_class->finalize = g_memory_output_stream_finalize;
+  gobject_class->set_property = g_memory_output_stream_set_property;
+  gobject_class->get_property = g_memory_output_stream_get_property;
+  gobject_class->finalize     = g_memory_output_stream_finalize;
 
   ostream_class = G_OUTPUT_STREAM_CLASS (klass);
 
@@ -125,6 +146,115 @@ g_memory_output_stream_class_init (GMemoryOutputStreamClass *klass)
   ostream_class->write_finish = g_memory_output_stream_write_finish;
   ostream_class->close_async  = g_memory_output_stream_close_async;
   ostream_class->close_finish = g_memory_output_stream_close_finish;
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_DATA,
+                                   g_param_spec_pointer ("data",
+                                                         P_("Data Buffer"),
+                                                         P_("Pointer to buffer where data will be written."),
+                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
+                                                         G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_SIZE,
+                                   g_param_spec_ulong ("size",
+                                                       P_("Data Buffer Size"),
+                                                       P_("Current size of the data buffer."),
+                                                       0, G_MAXULONG, 0,
+                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
+                                                       G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_DATA_SIZE,
+                                   g_param_spec_ulong ("data-size",
+                                                       P_("Data Size"),
+                                                       P_("Size of data written to the buffer."),
+                                                       0, G_MAXULONG, 0,
+                                                       G_PARAM_READABLE |
+                                                       G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_REALLOC_FUNCTION,
+                                   g_param_spec_pointer ("realloc-function",
+                                                         P_("Memory Reallocation Function"),
+                                                         P_("Function with realloc semantics called to enlarge the buffer."),
+                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
+                                                         G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_DESTROY_FUNCTION,
+                                   g_param_spec_pointer ("destroy-function",
+                                                         P_("Destroy Notification Function"),
+                                                         P_("Function called with the buffer as argument when the stream is destroyed."),
+                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
+                                                         G_PARAM_STATIC_STRINGS));
+}
+
+static void
+g_memory_output_stream_set_property (GObject      *object,
+                                     guint         prop_id,
+                                     const GValue *value,
+                                     GParamSpec   *pspec)
+{
+  GMemoryOutputStream        *stream;
+  GMemoryOutputStreamPrivate *priv;
+
+  stream = G_MEMORY_OUTPUT_STREAM (object);
+  priv = stream->priv;
+
+  switch (prop_id)
+    {
+    case PROP_DATA:
+      priv->data = g_value_get_pointer (value);
+      break;
+    case PROP_SIZE:
+      priv->len = g_value_get_ulong (value);
+      break;
+    case PROP_REALLOC_FUNCTION:
+      priv->realloc_fn = g_value_get_pointer (value);
+      break;
+    case PROP_DESTROY_FUNCTION:
+      priv->destroy = g_value_get_pointer (value);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+static void
+g_memory_output_stream_get_property (GObject      *object,
+                                     guint         prop_id,
+                                     GValue       *value,
+                                     GParamSpec   *pspec)
+{
+  GMemoryOutputStream        *stream;
+  GMemoryOutputStreamPrivate *priv;
+
+  stream = G_MEMORY_OUTPUT_STREAM (object);
+  priv = stream->priv;
+
+  switch (prop_id)
+    {
+    case PROP_DATA:
+      g_value_set_pointer (value, priv->data);
+      break;
+    case PROP_SIZE:
+      g_value_set_ulong (value, priv->len);
+      break;
+    case PROP_DATA_SIZE:
+      g_value_set_ulong (value, priv->valid_len);
+      break;
+    case PROP_REALLOC_FUNCTION:
+      g_value_set_pointer (value, priv->realloc_fn);
+      break;
+    case PROP_DESTROY_FUNCTION:
+      g_value_set_pointer (value, priv->destroy);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
 }
 
 static void
@@ -159,22 +289,24 @@ g_memory_output_stream_init (GMemoryOutputStream *stream)
   stream->priv = G_TYPE_INSTANCE_GET_PRIVATE (stream,
                                               G_TYPE_MEMORY_OUTPUT_STREAM,
                                               GMemoryOutputStreamPrivate);
+  stream->priv->pos = 0;
+  stream->priv->valid_len = 0;
 }
 
 /**
  * g_memory_output_stream_new:
  * @data: pointer to a chunk of memory to use, or %NULL
- * @len: the size of @data
- * @realloc_fn: a function with realloc() semantics to be called when 
+ * @size: the size of @data
+ * @realloc_function: a function with realloc() semantics to be called when
  *     @data needs to be grown, or %NULL
- * @destroy: a function to be called on @data when the stream is finalized,
- *     or %NULL
+ * @destroy_function: a function to be called on @data when the stream is
+ *     finalized, or %NULL
  *
- * Creates a new #GMemoryOutputStream. 
+ * Creates a new #GMemoryOutputStream.
  *
  * If @data is non-%NULL, the stream  will use that for its internal storage.
  * If @realloc_fn is non-%NULL, it will be used for resizing the internal
- * storage when necessary. To construct a fixed-size output stream, 
+ * storage when necessary. To construct a fixed-size output stream,
  * pass %NULL as @realloc_fn.
  * |[
  * /&ast; a stream that can grow &ast;/
@@ -189,24 +321,18 @@ g_memory_output_stream_init (GMemoryOutputStream *stream)
  **/
 GOutputStream *
 g_memory_output_stream_new (gpointer       data,
-                            gsize          len,
-                            GReallocFunc   realloc_fn,
-                            GDestroyNotify destroy)
+                            gsize          size,
+                            GReallocFunc   realloc_function,
+                            GDestroyNotify destroy_function)
 {
   GOutputStream *stream;
-  GMemoryOutputStreamPrivate *priv;
 
-  stream = g_object_new (G_TYPE_MEMORY_OUTPUT_STREAM, NULL);
-
-  priv = G_MEMORY_OUTPUT_STREAM (stream)->priv;
-
-  priv->data = data;
-  priv->len = len;
-  priv->realloc_fn = realloc_fn;
-  priv->destroy = destroy;
-
-  priv->pos = 0;
-  priv->valid_len = 0;
+  stream = g_object_new (G_TYPE_MEMORY_OUTPUT_STREAM,
+                         "data", data,
+                         "size", size,
+                         "realloc-function", realloc_function,
+                         "destroy-function", destroy_function,
+                         NULL);
 
   return stream;
 }
@@ -215,11 +341,11 @@ g_memory_output_stream_new (gpointer       data,
  * g_memory_output_stream_get_data:
  * @ostream: a #GMemoryOutputStream
  *
- * Gets any loaded data from the @ostream. 
+ * Gets any loaded data from the @ostream.
+ *
+ * Note that the returned pointer may become invalid on the next
+ * write or truncate operation on the stream.
  *
- * Note that the returned pointer may become invalid on the next 
- * write or truncate operation on the stream. 
- * 
  * Returns: pointer to the stream's data
  **/
 gpointer
@@ -245,14 +371,14 @@ g_memory_output_stream_get_data (GMemoryOutputStream *ostream)
  *
  * If you want the number of bytes currently written to the stream, use
  * g_memory_output_stream_get_data_size().
- * 
+ *
  * Returns: the number of bytes allocated for the data buffer
  */
 gsize
 g_memory_output_stream_get_size (GMemoryOutputStream *ostream)
 {
   g_return_val_if_fail (G_IS_MEMORY_OUTPUT_STREAM (ostream), 0);
-  
+
   return ostream->priv->len;
 }
 
@@ -263,7 +389,7 @@ g_memory_output_stream_get_size (GMemoryOutputStream *ostream)
  * Returns the number of bytes from the start up
  * to including the last byte written in the stream
  * that has not been truncated away.
- * 
+ *
  * Returns: the number of bytes written to the stream
  *
  * Since: 2.18
@@ -272,33 +398,14 @@ gsize
 g_memory_output_stream_get_data_size (GMemoryOutputStream *ostream)
 {
   g_return_val_if_fail (G_IS_MEMORY_OUTPUT_STREAM (ostream), 0);
-  
+
   return ostream->priv->valid_len;
 }
 
-
-static gboolean
-array_check_boundary (GMemoryOutputStream  *stream,
-                      goffset               size,
-                      GError              **error)
-{
-  if (size > G_MAXUINT)
-    {
-      g_set_error_literal (error,
-                           G_IO_ERROR,
-                           G_IO_ERROR_FAILED,
-                           _("Reached maximum data array limit"));
-
-      return FALSE;
-    }
-
-  return TRUE;
-}
-
 static gboolean
 array_resize (GMemoryOutputStream  *ostream,
               gsize                 size,
-	      gboolean              allow_partial,
+              gboolean              allow_partial,
               GError              **error)
 {
   GMemoryOutputStreamPrivate *priv;
@@ -307,18 +414,15 @@ array_resize (GMemoryOutputStream  *ostream,
 
   priv = ostream->priv;
 
-  if (!array_check_boundary (ostream, size, error))
-    return FALSE;
-
   if (priv->len == size)
     return TRUE;
 
   if (!priv->realloc_fn)
     {
       if (allow_partial &&
-	  priv->pos < priv->len)
-	return TRUE; /* Short write */
-      
+          priv->pos < priv->len)
+        return TRUE; /* Short write */
+
       g_set_error_literal (error,
                            G_IO_ERROR,
                            G_IO_ERROR_NO_SPACE,
@@ -329,12 +433,12 @@ array_resize (GMemoryOutputStream  *ostream,
   len = priv->len;
   data = priv->realloc_fn (priv->data, size);
 
-  if (size > 0 && !data) 
+  if (size > 0 && !data)
     {
       if (allow_partial &&
-	  priv->pos < priv->len)
-	return TRUE; /* Short write */
-      
+          priv->pos < priv->len)
+        return TRUE; /* Short write */
+
       g_set_error_literal (error,
                            G_IO_ERROR,
                            G_IO_ERROR_NO_SPACE,
@@ -347,10 +451,10 @@ array_resize (GMemoryOutputStream  *ostream,
 
   priv->data = data;
   priv->len = size;
-  
+
   if (priv->len < priv->valid_len)
     priv->valid_len = priv->len;
-  
+
   return TRUE;
 }
 
@@ -383,29 +487,52 @@ g_memory_output_stream_write (GOutputStream  *stream,
   if (count == 0)
     return 0;
 
-  if (priv->pos + count > priv->len) 
+  /* Check for address space overflow, but only if the buffer is resizable.
+     Otherwise we just do a short write and don't worry. */
+  if (priv->realloc_fn && priv->pos + count < priv->pos)
+    goto overflow;
+
+  if (priv->pos + count > priv->len)
     {
       /* At least enought to fit the write, rounded up
-	 for greater than linear growth */
+	     for greater than linear growth.
+         TODO: This wastes a lot of memory at large stream sizes.
+               Figure out a more rational allocation strategy. */
       new_size = g_nearest_pow (priv->pos + count);
+      /* Check for overflow again. We have only checked if
+         pos + count > G_MAXSIZE, but it only catches the case of writing
+         more than 4GiB total on a 32-bit system. There's still the problem
+         of g_nearest_pow overflowing above 0x7fffffff, so we're
+         effectively limited to 2GiB. */
+      if (new_size < priv->len)
+        goto overflow;
+
       new_size = MAX (new_size, MIN_ARRAY_SIZE);
-      
       if (!array_resize (ostream, new_size, TRUE, error))
-	return -1;
+        return -1;
     }
 
   /* Make sure we handle short writes if the array_resize
      only added part of the required memory */
   count = MIN (count, priv->len - priv->pos);
-  
+
   dest = (guint8 *)priv->data + priv->pos;
-  memcpy (dest, buffer, count); 
+  memcpy (dest, buffer, count);
   priv->pos += count;
-  
+
   if (priv->pos > priv->valid_len)
     priv->valid_len = priv->pos;
 
   return count;
+
+ overflow:
+  /* Overflow: buffer size would need to be bigger than G_MAXSIZE. */
+  g_set_error_literal (error,
+                       G_IO_ERROR,
+                       G_IO_ERROR_NO_SPACE,
+                       _("Amount of memory required to process the write is "
+                         "larger than available address space"));
+  return -1;
 }
 
 static gboolean
@@ -433,14 +560,13 @@ g_memory_output_stream_write_async (GOutputStream       *stream,
                                            count,
                                            cancellable,
                                            NULL);
- 
 
   simple = g_simple_async_result_new (G_OBJECT (stream),
                                       callback,
                                       data,
                                       g_memory_output_stream_write_async);
-  
-  g_simple_async_result_set_op_res_gssize (simple, nwritten); 
+
+  g_simple_async_result_set_op_res_gssize (simple, nwritten);
   g_simple_async_result_complete_in_idle (simple);
   g_object_unref (simple);
 }
@@ -454,8 +580,8 @@ g_memory_output_stream_write_finish (GOutputStream  *stream,
   gssize nwritten;
 
   simple = G_SIMPLE_ASYNC_RESULT (result);
-  
-  g_warn_if_fail (g_simple_async_result_get_source_tag (simple) == 
+
+  g_warn_if_fail (g_simple_async_result_get_source_tag (simple) ==
                   g_memory_output_stream_write_async);
 
   nwritten = g_simple_async_result_get_op_res_gssize (simple);
@@ -480,7 +606,7 @@ g_memory_output_stream_close_async (GOutputStream       *stream,
 
   /* will always return TRUE */
   g_memory_output_stream_close (stream, cancellable, NULL);
-  
+
   g_simple_async_result_complete_in_idle (simple);
   g_object_unref (simple);
 }
@@ -494,7 +620,7 @@ g_memory_output_stream_close_finish (GOutputStream  *stream,
 
   simple = G_SIMPLE_ASYNC_RESULT (result);
 
-  g_warn_if_fail (g_simple_async_result_get_source_tag (simple) == 
+  g_warn_if_fail (g_simple_async_result_get_source_tag (simple) ==
                   g_memory_output_stream_close_async);
 
   return TRUE;
@@ -532,7 +658,7 @@ g_memory_output_stream_seek (GSeekable    *seekable,
   stream = G_MEMORY_OUTPUT_STREAM (seekable);
   priv = stream->priv;
 
-  switch (type) 
+  switch (type)
     {
     case G_SEEK_CUR:
       absolute = priv->pos + offset;
@@ -545,7 +671,7 @@ g_memory_output_stream_seek (GSeekable    *seekable,
     case G_SEEK_END:
       absolute = priv->len + offset;
       break;
-  
+
     default:
       g_set_error_literal (error,
                            G_IO_ERROR,
@@ -555,17 +681,23 @@ g_memory_output_stream_seek (GSeekable    *seekable,
       return FALSE;
     }
 
-  if (absolute < 0) 
+  if (absolute < 0)
     {
       g_set_error_literal (error,
                            G_IO_ERROR,
                            G_IO_ERROR_INVALID_ARGUMENT,
-                           _("Invalid seek request"));
+                           _("Requested seek before the beginning of the stream"));
       return FALSE;
     }
 
-  if (!array_check_boundary (stream, absolute, error)) 
-    return FALSE;  
+  if (absolute > priv->len)
+    {
+      g_set_error_literal (error,
+                           G_IO_ERROR,
+                           G_IO_ERROR_INVALID_ARGUMENT,
+                           _("Requested seek beyond the end of the stream"));
+      return FALSE;
+    }
 
   priv->pos = absolute;
 
@@ -591,7 +723,7 @@ g_memory_output_stream_truncate (GSeekable     *seekable,
                                  GError       **error)
 {
   GMemoryOutputStream *ostream = G_MEMORY_OUTPUT_STREAM (seekable);
- 
+
   if (!array_resize (ostream, offset, FALSE, error))
     return FALSE;
 
